
Include "data.geo";

Group{
  Domaine = Region[{AIR,AIRGAP,FERRITE,BOBINEA,BOBINEB,BOBINEC,BOITEINF}];
  Domaine_NonMag = Region[{AIR,AIRGAP,BOBINEA,BOBINEB,BOBINEC,BOITEINF}];
  Ferrite = Region[{FERRITE}];
  BobineY = Region[{BOBINEA}];
  BobineX = Region[{BOBINEB}];
  BobineCoude = Region[{BOBINEC}];
  Bobines = Region[{BOBINEA,BOBINEB,BOBINEC}];
  BoiteInf = Region[{BOITEINF}];
  Dirichlet = Region[{BORDS}];
}


Function{		
  If(NonLineaire)
    Mat_core_b = {0.000000, 0.068851, 0.079231, 0.096630, 0.104285, 0.117920, 0.123374, 
      0.150912, 0.191749, 0.202635, 0.239024, 0.243936, 0.245556, 0.267244, 
      0.290565, 0.296190, 0.312896, 0.313356, 0.335942, 0.364839, 0.368482, 
      0.376766, 0.477117, 0.494881, 0.666990, 0.828539, 0.832720};

    Mat_core_h = {0.000000, 44.635198, 47.996938, 54.360481, 57.254136, 63.513926, 66.205407, 
      83.184012, 128.638642, 150.321852, 280.295679, 307.761975, 317.501481, 
      486.037901, 775.608272, 869.477778, 1359.072840, 1373.269136, 2431.451852, 
      5699.176543, 6389.154321, 8458.007407, 61532.617284, 73419.641975, 
      197003.703704, 321027.901235, 324350.000000};

    Mat_core_b2 = Mat_core_b()^2;
    Mat_core_h2 = Mat_core_h()^2;
    Mat_core_nu = Mat_core_h() / Mat_core_b();
    Mat_core_nu(0) = Mat_core_nu(1);
    Mat_core_nu_b2  = ListAlt[Mat_core_b2(), Mat_core_nu()] ;
    nu_core[] = InterpolationLinear[ SquNorm[$1] ]{ Mat_core_nu_b2() } ;
    dnudb2_core[] = dInterpolationLinear[SquNorm[$1]]{ Mat_core_nu_b2() } ;
    h_core[] = nu_core[$1] * $1 ;
    dhdb_core[] = TensorDiag[1,1,1] * nu_core[$1#1] +
    2*dnudb2_core[#1] * SquDyadicProduct[#1]  ;
    nu[ Ferrite ]  = nu_core[$1];
    dhdb[ Ferrite ] = dhdb_core[$1];
    Else
    nu[ Ferrite ]  = 1./(mur*mu0);
  EndIf
  
  nu[ Domaine_NonMag ] = 1./mu0;
  theta[] = Atan2[Y[],X[]];
  jdir[ BobineY] = Vector[0,1,0];
  jdir[ BobineX] = Vector[-1,0,0];
  jdir[ BobineCoude ] = Vector[-Sin[theta[]],Cos[theta[]],0];
  js[ Bobines ] = N*I/Sb*jdir[]; //Vector[0,0,N*I/Sb];
}

Jacobian{
  { Name Jvol;
    Case {
      { Region BoiteInf ; Jacobian VolSphShell{Ri,Re} ;}
      { Region All ; Jacobian Vol ; }
    }
  }
}


Integration {
  { Name Integ;
    Case { 
      {Type Gauss;
	Case { 
	  { GeoElement Tetrahedron ; NumberOfPoints 5; }
	  { GeoElement Prism ; NumberOfPoints 6; }
	  { GeoElement Pyramid ; NumberOfPoints 8; }
	  { GeoElement Hexahedron ; NumberOfPoints 6; }	
	}
      }
    }
  }
}

Constraint {	
  {Name Dirichlet ;
    Case {      
      { Region Dirichlet ; Type Assign ; Value 0. ; }
    }
  }
  If (Jauge_arbre)
    { Name Jauge_arbre ; Type Assign ;
      Case {
	{Region Domaine ; SubRegion Dirichlet ; Value 0. ; }
      }
    }
  EndIf
}

FunctionSpace {
  { Name Hrot ; Type Form1 ; 
    BasisFunction { 
      { Name se ; NameOfCoef ae ; Function BF_Edge ; 
	Support Region[{Domaine}]  ; Entity EdgesOf[All] ; }  } 
    Constraint {
      { NameOfCoef ae  ; EntityType EdgesOf ; NameOfConstraint Dirichlet ; }
      If (Jauge_arbre)
	{ NameOfCoef ae  ; EntityType EdgesOfTreeIn ; EntitySubType StartingOn ;
	  NameOfConstraint Jauge_arbre ; }
      EndIf
    }
  }  
  If (Jauge_arbre == 0)
    { Name Hgrad ; Type Form0 ; 
      BasisFunction { 
	{ Name sn ; NameOfCoef xin ; Function BF_Node ; 
	  Support Region[{Domaine}]  ; Entity NodesOf[All] ; }  } 
      Constraint {
	{ NameOfCoef xin  ; EntityType NodesOf ; NameOfConstraint Dirichlet ; }
      }
    } 
  EndIf
}


Formulation {
  { Name Magnetostat ; Type FemEquation ;
    Quantity {
      { Name a  ; Type Local  ; NameOfSpace Hrot ; }
      If (Jauge_arbre == 0)
	{Name xi ; Type Local  ; NameOfSpace Hgrad ; }
      EndIf
    }
    Equation {
      Integral { [ nu[] * Dof{d a} , {d a} ];
	In Domaine_NonMag; Jacobian Jvol; Integration Integ; }

      If(NonLineaire)
	Integral { [ nu[{d a}] * {d a} , {d a} ];
	  In Ferrite; Jacobian Jvol; Integration Integ; }
	
	Integral { [ dhdb[{d a}] * Dof{d a} , {d a} ];
	  In Ferrite; Jacobian Jvol; Integration Integ; }
	
	Integral { [ - dhdb[{d a}] * {d a} , {d a} ];
	  In Ferrite; Jacobian Jvol; Integration Integ; }
	
	Else
	
	Integral { [ nu[{d a}] * Dof{d a}, {d a} ];
	  In Ferrite; Jacobian Jvol; Integration Integ; }
	
      EndIf
      
      Integral { [ -js[] , {a} ];
	In Bobines; Jacobian Jvol; Integration Integ; }
      
      If (Jauge_arbre == 0)
	Integral{ [ Dof{Grad xi} , {a}  ];
	  In Domaine; Jacobian Jvol; Integration Integ; }
	
	Integral { [ Dof{a} , {Grad xi} ];
	  In Domaine; Jacobian Jvol; Integration Integ; }
      EndIf
    }
  } 
}

Resolution {
  { Name analyse ;
    System {
      { Name A ; NameOfFormulation Magnetostat ; }
    }
    Operation {
      InitSolution[A];
      Generate[A]; 
      Solve[A];
      
      If(NonLineaire)
	Generate[A]; GetResidual[A, $res0];
	Evaluate[ $res = $res0, $iter = 0 ];
	Print[{$iter, $res, $res / $res0},
	  Format "Residual %03g: abs %14.12e rel %14.12e"];
	
	While[$res > NL_tol_abs && $res / $res0 > NL_tol_rel &&
	  $res / $res0 <= 1 && $iter < NL_iter_max]{
	  Solve[A]; Generate[A]; GetResidual[A, $res];
	  Evaluate[ $iter = $iter + 1 ];
	  Print[{$iter, $res, $res / $res0},
	    Format "Residual %03g: abs %14.12e rel %14.12e"];
	}
	
      EndIf
      SaveSolution[A];
    }
  }
}

  PostProcessing {
    { Name PostProc ; NameOfFormulation Magnetostat ;
      Quantity {	
	{ Name a  ; Value { Term { [ {a} ] ; 
	      In Domaine ; Jacobian Jvol ; } } }
	{ Name js  ; Value { Term { [ js[] ] ; 
	      In Bobines ; Jacobian Jvol ; } } }
	{ Name b  ; Value { Term { [ {Curl a} ] ; 
	      In Domaine ; Jacobian Jvol ; } } }
	{ Name norm_b  ; Value { Term { [ Norm[{Curl a}] ] ; 
	      In Domaine ; Jacobian Jvol ; } } }
	{ Name h  ; Value { Term { [ nu[{Curl a}]*{Curl a} ] ; 
	      In Domaine ; Jacobian Jvol ; } } }
	{ Name L ; Value { Integral { [ 1e6*4*N*jdir[]*{a}/(Sb/2)/I ]  ; 
	      In Bobines ;  Jacobian Jvol ; Integration Integ ;} } } 
      }
    }
  }

  PostOperation PostOp UsingPost PostProc {
    Print[ a, OnElementsOf Domaine, File "a.pos"] ;
    Print[ js, OnElementsOf Bobines, File "js.pos"] ;
    Print[ b, OnElementsOf Domaine, File "b.pos"] ;
    Print[ norm_b, OnElementsOf Ferrite, Smoothing, File "normb.pos"] ;
    Print[ h, OnElementsOf Domaine, File "h.pos"] ;
    Print[ L[Bobines], OnGlobal, File "inductance.dat", 
      SendToServer "Modele/02Resultats/2Inductance (µH)", Color "AliceBlue" ]; 
  }

